.legend.leaflet-control i{
    float: left;
    text-align: left;
    height: 13px;
    line-height: 13px;
    margin-right: 8px;
    opacity: 0.7;
    width: 18px;
}

.info.leaflet-control{
    background: none repeat scroll 0 0 rgba(255, 255, 255, 0.8);
    border-radius: 5px 5px 5px 5px;
    box-shadow: 0 0 15px rgba(0, 0, 0, 0.2);
    padding: 6px 8px;
}